package io.gitlab.jfronny.commons.logger;

import io.gitlab.jfronny.commons.ref.WeakSet;
import org.jetbrains.annotations.Nullable;
import org.slf4j.ILoggerFactory;
import org.slf4j.Logger;
import org.slf4j.event.Level;

import java.util.Objects;
import java.util.Set;

/**
 * A logger factory that allows apps to swap the backing factory at runtime, automatically updating all created loggers.
 */
public class HotswapLoggerFactory implements LeveledLoggerFactory {
    private static final Set<HotswapLoggerFactory> KNOWN_INSTANCES = new WeakSet<>();

    /**
     * Resets all known instances to use the default logger factory.
     */
    public static void resetAllStrategies() {
        for (HotswapLoggerFactory factory : KNOWN_INSTANCES) {
            factory.resetStrategy();
        }
    }

    /**
     * Updates all known instances to use the new logger factory.
     * @param factory the new logger factory to use for all known instances
     */
    public static void updateAllStrategies(ILoggerFactory factory) {
        for (HotswapLoggerFactory f : KNOWN_INSTANCES) {
            f.updateStrategy(factory);
        }
    }

    /**
     * Updates all known instances to use the new logger factory.
     * @param factory the new logger factory to use for all known instances
     */
    public static void updateAllStrategies(LeveledLoggerFactory factory) {
        for (HotswapLoggerFactory f : KNOWN_INSTANCES) {
            f.updateStrategy(factory);
        }
    }

    /**
     * Updates all known instances to use the new minimum log level if supported.
     * @param minimumLevel the new minimum level to use for all known instances
     */
    public static void updateAllDefaultLevels(Level minimumLevel) {
        for (HotswapLoggerFactory f : KNOWN_INSTANCES) {
            f.updateDefaultLevel(minimumLevel);
        }
    }

    public HotswapLoggerFactory() {
        KNOWN_INSTANCES.add(this);
    }

    private LeveledLoggerFactory defaultFactory = StdoutLogger::fancy;
    private Level minimumLevel = Level.INFO;
    private int version = 0;

    /**
     * Resets the strategy to use the default logger factory.
     */
    public void resetStrategy() {
        updateStrategy(StdoutLogger::fancy);
    }

    /**
     * Updates the strategy to use the new logger factory.
     * @param factory the new logger factory to use
     */
    public void updateStrategy(ILoggerFactory factory) {
        if (factory instanceof LeveledLoggerFactory f) {
            updateStrategy(f);
        } else {
            updateStrategy((name, level) -> factory.getLogger(name));
        }
    }

    /**
     * Updates the strategy to use the new logger factory.
     * @param factory the new logger factory to use
     */
    public void updateStrategy(LeveledLoggerFactory factory) {
        synchronized (this) {
            defaultFactory = Objects.requireNonNull(factory);
            version++;
        }
    }

    /**
     * Updates the minimum log level to use if supported.
     * @param minimumLevel the new minimum level to use if supported
     */
    public void updateDefaultLevel(Level minimumLevel) {
        synchronized (this) {
            this.minimumLevel = Objects.requireNonNull(minimumLevel);
            version++;
        }
    }

    @Override
    public Logger getLogger(String name, @Nullable Level level) {
        return new SwappableDelegateLogger(name, level);
    }

    private class SwappableDelegateLogger extends DelegateLogger {
        private final String name;
        private final boolean keepLevel;
        private Level lastMinimumLevel;
        private Logger delegate;
        int version = -1;

        private SwappableDelegateLogger(String name, @Nullable Level level) {
            this.name = name == null ? "null" : name;
            this.keepLevel = level != null;
            this.lastMinimumLevel = level == null ? HotswapLoggerFactory.this.minimumLevel : level;
            updateStrategy();
        }

        public void updateStrategy() {
            if (this.version == HotswapLoggerFactory.this.version) return;
            synchronized (HotswapLoggerFactory.this) {
                this.version = HotswapLoggerFactory.this.version;
                if (!keepLevel) this.lastMinimumLevel = HotswapLoggerFactory.this.minimumLevel;
                this.delegate = HotswapLoggerFactory.this.defaultFactory.getLogger(this.name, this.lastMinimumLevel);
            }
        }

        @Override
        protected Logger getDelegate() {
            updateStrategy();
            return delegate;
        }
    }
}
